/**
 * Created by hyu.Yale on 2015/10/26.
 */

function AssetsPieDrawer(el) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }

    if (!this.canvas) {
        return;
    }

    this.ctx = this.canvas.getContext("2d");
    this.mWidth = document.body.clientWidth * 0.96;
    this.mHeight = this.canvas.height;

    this.mDiameter = this.mModelDiameter * this.canvas.width / this.mModelBodyWidth;
    this.mSingleGap = this.mModelSingleGap * this.canvas.width / this.mModelBodyWidth - 5;
    this.mRingWidth = 35;

    this.mCenterX = this.mWidth / 2
    this.mCenterY = this.mSingleGap + this.mDiameter / 2;

    var ctx = this.ctx,
        width = this.mWidth,
        height = this.mHeight;

    if (window.devicePixelRatio) {
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
        this.canvas.height = height * window.devicePixelRatio;
        this.canvas.width = width * window.devicePixelRatio;
        ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
    }
}

AssetsPieDrawer.prototype = {
    mWidth: 0,//Canvas宽度
    mHeight: 0,//Canvas 高度
    mCenterX: 0,//圆心X
    mCenterY: 0,//圆心Y
    mSingleGap: 0,//上下间距
    mDiameter: 0,//圆直径
    mRingWidth: 0,//环宽度
    mBodyWidth: 0,//饼图主体内容宽度
    mBodyHeight: 0,//饼图主体内容宽度
    mDataSourceCount: 3,//饼图主体内容宽度
    mScreenWidth: 320,//基于屏幕760宽度做的图
    mDataModel: [],
    mItemTxtHeight: 25,

    mModelSingleGap: 30,//效果图上下间距
    mModelDiameter: 382,//效果图圆直径
    mModelRingWidth: 60,//效果图环宽度
    mModelBodyWidth: 688,//效果图饼图主体内容宽度
    mModelBodyHeight: 462,//效果图饼图主体内容宽度
    mModelScreenWidth: 760,//效果图基于屏幕760宽度做的图
    version: "1.0.1",
    color: "#DFDFDF",
    drawArc: function () {
        var dataModel = this.mDataModel;
        var totalValue = 0;
        $.each(dataModel, function (index, el) {
            if (!isNaN(el.value)) {
                totalValue += parseFloat(el.value);
            }
        });

        var start = 3 * Math.PI / 2;
        var end = 2 * Math.PI * 0.85;
        var ctx = this.ctx;

        var centerX = this.mCenterX;
        var centerY = this.mCenterY;
        var radius1 = this.mDiameter / 2 - this.mRingWidth / 2 - 2;
        var radius2 = this.mDiameter / 2;
        var ringWidth = this.mRingWidth;

        $.each(dataModel, function (index, el) {
            var rate = el.value / totalValue;
            el.rateTxt = fixStringRate(fixNum(rate * 100, 2));

            //解决老版本以及4.0模拟器上的单一数据源的时候
            //圆绘制不出来的bug。
            if (rate == 1) {
                rate = 0.99999;
            }
            end = start + 2 * Math.PI * rate;
            ctx.beginPath();
            ctx.arc(centerX, centerY, radius1, start, end, false);
            ctx.lineWidth = ringWidth;
            ctx.strokeStyle = el.color;
            ctx.stroke();

            ctx.beginPath();
            ctx.arc(centerX, centerY, radius2, start, end, false);
            ctx.lineWidth = 1;
            ctx.stroke();
            start += 2 * Math.PI * rate;
        });
    },

    drawSubTxt: function () {
        var context = this.ctx;

        var itemHeight = this.mItemTxtHeight;
        var canvasWidth = this.mWidth;
        //矩形宽度
        var rectWidth = 10;
        //文字距离矩形的间距
        var rectTxtMargin = 3;

        //距离左右边界间距
        var borderMargin = 10;
        //单行文字2列之间的距离
        var centerMargin = 20;
        //剩下的总长度
        var totalLeftWidth = canvasWidth - 2 * borderMargin - centerMargin;
        var singleColumnWidth = totalLeftWidth / 2;

        var maxLeftColumnTxtWidth = 0;
        var maxRightColumnTxtWidth = 0;

        //中间变量，用于存储可用文字宽度
        var cellWidth = 0;
        //左边绘制宽度，包含矩形以及间距等
        var leftRectWidth = 0;
        //右边绘制宽度，包含矩形以及间距等
        var rightRectWidth = 0;

        var startX = 0;
        var startY = 195;
        context.font = "15px Arial";
        context.fillStyle = "#999";

        //找出左右列最长的宽度
        $.each(this.mDataModel, function (index, el) {
            var rateTxt = "(" + el.rateTxt + ")";
            var totalDrawTxt = fixString(el.subTxt);
            var measuredWidth = context.measureText(totalDrawTxt + rateTxt).width;
            if (index % 2 == 0) {
                if (measuredWidth > maxLeftColumnTxtWidth) {
                    maxLeftColumnTxtWidth = measuredWidth;
                }
            } else {
                if (measuredWidth > maxRightColumnTxtWidth) {
                    maxRightColumnTxtWidth = measuredWidth;
                }
            }
        });

        var maxLeftTotalWidth = maxLeftColumnTxtWidth + rectWidth + rectTxtMargin;
        var maxRightTotalWidth = (0 == maxRightColumnTxtWidth ? 0 : (maxRightColumnTxtWidth + rectWidth + rectTxtMargin));

        //2列都够绘制，最优
        if (maxLeftTotalWidth <= singleColumnWidth && maxRightTotalWidth <= singleColumnWidth) {
            leftRectWidth = maxLeftTotalWidth;
            rightRectWidth = maxRightTotalWidth;
            if (this.mDataModel.length == 1) {
                var el = this.mDataModel[0];

                var rateTxt = "(" + el.rateTxt + ")";
                var ellipsis = "...";
                var totalDrawTxt = fixString(el.subTxt);
                var measuredRateTxt = context.measureText(rateTxt);
                context.fillStyle = "#999";
                //文字太长
                if (context.measureText(totalDrawTxt + rateTxt).width > (canvasWidth - 2 * borderMargin)) {
                    var tempTxt = findMaxTxt(canvasWidth - 2 * borderMargin - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                    if (tempTxt != totalDrawTxt) {
                        tempTxt += "..." + rateTxt;
                    }

                    startX = borderMargin;
                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);
                    startX += rectWidth + rectTxtMargin;

                    context.fillText(tempTxt, startX, startY + 10);
                } else {
                    startX = (canvasWidth - maxLeftTotalWidth) / 2;
                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);

                    startX += rectWidth + rectTxtMargin;
                    context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                }
            } else {
                $.each(this.mDataModel, function (index, el) {
                    var isLeft = false;
                    if ((index % 2) == 0) {
                        isLeft = true;
                        if (index != 0) {
                            startY += itemHeight;
                        }
                    }

                    if (isLeft) {
                        startX = (canvasWidth - leftRectWidth - rightRectWidth - centerMargin) / 2;
                        cellWidth = leftRectWidth - rectWidth - rectTxtMargin;
                    } else {
                        startX = (canvasWidth - leftRectWidth - rightRectWidth - centerMargin) / 2 + leftRectWidth + centerMargin;
                        cellWidth = rightRectWidth - rectWidth - rectTxtMargin;
                    }

                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);

                    startX += rectWidth + rectTxtMargin;

                    var rateTxt = "(" + el.rateTxt + ")";
                    var ellipsis = "...";
                    var totalDrawTxt = fixString(el.subTxt);
                    var measuredRateTxt = context.measureText(rateTxt);
                    context.fillStyle = "#999";
                    //文字太长
                    if (context.measureText(totalDrawTxt + rateTxt).width > cellWidth) {
                        var tempTxt = findMaxTxt(cellWidth - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                        if (tempTxt != totalDrawTxt) {
                            tempTxt += "..." + rateTxt;
                        }
                        context.fillText(tempTxt, startX, startY + 10);
                    } else {
                        context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                    }
                });
            }
            return;
        }

        if (maxLeftTotalWidth > singleColumnWidth && maxRightTotalWidth > singleColumnWidth) {
            if (this.mDataModel.length == 1) {
                //var el = this.mDataModel[0];
                //
                //var rateTxt = "(" + el.rateTxt + ")";
                //var ellipsis = "...";
                //var totalDrawTxt = fixString(el.subTxt);
                //var measuredRateTxt = context.measureText(rateTxt);
                //context.fillStyle = "#999";
                ////文字太长
                //if (context.measureText(totalDrawTxt + rateTxt).width > (canvasWidth - 2 * borderMargin)) {
                //    var tempTxt = findMaxTxt(canvasWidth - 2 * borderMargin - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                //    if (tempTxt != totalDrawTxt) {
                //        tempTxt += "..." + rateTxt;
                //    }
                //
                //    startX = borderMargin;
                //    context.fillStyle = el.color;
                //    context.fillRect(startX, startY, rectWidth, rectWidth);
                //    startX += rectWidth + rectTxtMargin;
                //
                //    context.fillText(tempTxt, startX, startY + 10);
                //} else {
                //    startX = (canvasWidth - maxLeftTotalWidth) / 2;
                //    context.fillStyle = el.color;
                //    context.fillRect(startX, startY, rectWidth, rectWidth);
                //
                //    startX += rectWidth + rectTxtMargin;
                //    context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                //}
            } else {
                $.each(this.mDataModel, function (index, el) {
                    var isLeft = false;
                    if ((index % 2) == 0) {
                        isLeft = true;
                        if (index != 0) {
                            startY += itemHeight;
                        }
                    }

                    if (isLeft) {
                        startX = borderMargin;
                        cellWidth = singleColumnWidth - rectWidth - rectTxtMargin;
                    } else {
                        startX = borderMargin + singleColumnWidth + centerMargin;
                        cellWidth = singleColumnWidth - rectWidth - rectTxtMargin;
                    }

                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);

                    startX += rectWidth + rectTxtMargin;
                    var rateTxt = "(" + el.rateTxt + ")";
                    var ellipsis = "...";
                    var totalDrawTxt = fixString(el.subTxt);
                    var measuredRateTxt = context.measureText(rateTxt);
                    context.fillStyle = "#999";
                    //文字太长
                    if (context.measureText(totalDrawTxt + rateTxt).width > cellWidth) {
                        var tempTxt = findMaxTxt(cellWidth - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                        if (tempTxt != totalDrawTxt) {
                            tempTxt += "..." + rateTxt;
                        }
                        context.fillText(tempTxt, startX, startY + 10);
                    } else {
                        context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                    }
                });
            }
            return;
        }

        //总长度还够，尽量保证显示更多的文字
        if ((maxLeftTotalWidth + maxRightTotalWidth) <= totalLeftWidth) {
            var temLeftWidth = (totalLeftWidth - maxLeftTotalWidth - maxRightTotalWidth) / 2;
            $.each(this.mDataModel, function (index, el) {
                var isLeft = false;
                if ((index % 2) == 0) {
                    isLeft = true;
                    if (index != 0) {
                        startY += itemHeight;
                    }
                }
                if (isLeft) {
                    startX = temLeftWidth + borderMargin;
                    cellWidth = maxLeftTotalWidth - rectWidth - rectTxtMargin;
                } else {
                    startX = borderMargin + temLeftWidth + maxLeftTotalWidth + centerMargin;
                    cellWidth = maxRightTotalWidth - rectWidth - rectTxtMargin;
                }

                context.fillStyle = el.color;
                context.fillRect(startX, startY, rectWidth, rectWidth);

                startX += rectWidth + rectTxtMargin;

                var rateTxt = "(" + el.rateTxt + ")";
                var ellipsis = "...";
                var totalDrawTxt = fixString(el.subTxt);
                var measuredRateTxt = context.measureText(rateTxt);
                context.fillStyle = "#999";
                //文字太长
                if (context.measureText(totalDrawTxt + rateTxt).width > cellWidth) {
                    var tempTxt = findMaxTxt(cellWidth - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                    if (tempTxt != totalDrawTxt) {
                        tempTxt += "..." + rateTxt;
                    }
                    context.fillText(tempTxt, startX, startY + 10);
                } else {
                    context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                }
            });
            return;
        } else {
            context.font = "15px Arial";
            context.fillStyle = "#999";
            leftRectWidth = singleColumnWidth;
            rightRectWidth = singleColumnWidth;
            //左边大，右边必然是小于singleColumnWidth的，如果都大或者都小则会走上面的双判断流程
            if (maxLeftTotalWidth > singleColumnWidth) {
                $.each(this.mDataModel, function (index, el) {
                    var isLeft = false;
                    if ((index % 2) == 0) {
                        isLeft = true;
                        if (index != 0) {
                            startY += itemHeight;
                        }
                    }
                    if (isLeft) {
                        startX = borderMargin;
                        cellWidth = totalLeftWidth - maxRightTotalWidth - rectTxtMargin - rectWidth;
                    } else {
                        startX = canvasWidth - borderMargin - maxRightTotalWidth;
                        cellWidth = maxRightTotalWidth - rectTxtMargin - rectWidth;
                    }

                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);

                    startX += rectWidth + rectTxtMargin;

                    var rateTxt = "(" + el.rateTxt + ")";
                    var ellipsis = "...";
                    var totalDrawTxt = fixString(el.subTxt);
                    var measuredRateTxt = context.measureText(rateTxt);
                    context.fillStyle = "#999";
                    //文字太长
                    if (context.measureText(totalDrawTxt + rateTxt).width > cellWidth) {
                        var tempTxt = findMaxTxt(cellWidth - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                        if (tempTxt != totalDrawTxt) {
                            tempTxt += "..." + rateTxt;
                        }
                        context.fillText(tempTxt, startX, startY + 10);
                    } else {
                        context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                    }
                });
            }
            if (maxRightTotalWidth > singleColumnWidth) {
                $.each(this.mDataModel, function (index, el) {
                    var isLeft = false;
                    if ((index % 2) == 0) {
                        isLeft = true;
                        if (index != 0) {
                            startY += itemHeight;
                        }
                    }
                    if (isLeft) {
                        startX = borderMargin;
                        cellWidth = maxLeftTotalWidth - rectWidth - rectTxtMargin;
                    } else {
                        startX = borderMargin + maxLeftTotalWidth + centerMargin;
                        cellWidth = totalLeftWidth - maxLeftTotalWidth - rectTxtMargin - rectWidth;
                    }

                    context.fillStyle = el.color;
                    context.fillRect(startX, startY, rectWidth, rectWidth);

                    startX += rectWidth + rectTxtMargin;

                    var rateTxt = "(" + el.rateTxt + ")";
                    var ellipsis = "...";
                    var totalDrawTxt = fixString(el.subTxt);
                    var measuredRateTxt = context.measureText(rateTxt);
                    context.fillStyle = "#999";
                    //文字太长
                    if (context.measureText(totalDrawTxt + rateTxt).width > cellWidth) {
                        var tempTxt = findMaxTxt(cellWidth - measuredRateTxt.width - context.measureText(ellipsis).width, totalDrawTxt, context);
                        if (tempTxt != totalDrawTxt) {
                            tempTxt += "..." + rateTxt;
                        }
                        context.fillText(tempTxt, startX, startY + 10);
                    } else {
                        context.fillText(totalDrawTxt + rateTxt, startX, startY + 10);
                    }
                });
            }
        }


    },

    drawCenterTxt: function (totalAsset) {
        var txt = "总资产";
        var money = totalAsset;
        var context = this.ctx;
        var txtVerticalMargin = 10;
        context.font = "13px Arial";
        var measuredObj1 = this.ctx.measureText(txt);
        context.font = "15px Arial";
        var measuredObj2 = this.ctx.measureText(money);

        context.fillStyle = "#666";
        context.font = "13px Arial";
        var startX = this.mCenterX - measuredObj1.width / 2;
        var startY = this.mCenterY - txtVerticalMargin / 2;
        context.fillText(txt, startX, startY);

        context.font = "15px Arial";
        startX = this.mCenterX - measuredObj2.width / 2;
        startY = this.mCenterY + txtVerticalMargin;
        context.fillText(money, startX, startY + 6);
    },

    draw: function (dataModel, totalAsset) {
        this.mDataModel = dataModel;
        if (dataModel && dataModel.length > 0) {
            var totalItem = 0 == (dataModel.length % 2) ? parseInt(dataModel.length / 2) : parseInt(dataModel.length / 2) + 1;
            var totalSubTxtHeight = this.mItemTxtHeight * totalItem;
            this.mHeight = 185 + totalSubTxtHeight;
            $(".assets-canvas-center").css("height", this.mHeight + 'px');

            var ctx = this.ctx,
                width = this.mWidth,
                height = this.mHeight;
            if (window.devicePixelRatio) {
                this.canvas.style.width = width + "px";
                this.canvas.style.height = height + "px";
                this.canvas.height = height * window.devicePixelRatio;
                this.canvas.width = width * window.devicePixelRatio;
                ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
            }
            this.drawArc();
            this.drawSubTxt();
            this.drawCenterTxt(totalAsset);
        } else {
            this.drawEmpty();
        }
    },
    drawEmpty: function () {
        this.ctx.font = "15px Arial";
        this.ctx.fillStyle = "#999";
        var drawTxt = "暂无数据";
        var measured = this.ctx.measureText(drawTxt);
        this.ctx.fillText(drawTxt, (this.mWidth - measured.width) / 2, this.mHeight / 2, measured.width);
    },
    clear: function () {
        this.ctx.clearRect(0, 0, this.mWidth, this.mHeight);
        //解决某些机子上 不能清屏的BUG
        this.ctx.fillStyle = "#FFF";
        this.ctx.fillRect(0, 0, this.width, this.height);
    },

}