/**
 * Created by hyu.Yale on 2015/10/26.
 */

function ScaleDrawer(el) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }
    if (!this.canvas) {
        return;
    }
    this.ctx = this.canvas.getContext("2d");
    this.width = document.body.clientWidth * 0.9;
    this.height = this.canvas.height;
    var ctx = this.ctx,
        width = this.width,
        height = this.height;

    if (window.devicePixelRatio) {
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
        this.canvas.height = height * window.devicePixelRatio;
        this.canvas.width = width * window.devicePixelRatio;
        ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
    }
}

ScaleDrawer.prototype = {
    mWidth: 0,//Canvas宽度
    mHeight: 0,//Canvas 高度
    topTouchItemHeight: 25,//触摸显示高度
    mBarCount: 4,
    mTopPadding: 40,
    mBottomPadding: 30,
    mLeftRightPadding: 10,
    mTopExtra: 10,
    mBarMargin: 20,
    color: "#DFDFDF",
    dataList: [],
    trendList: [],
    mMax: 0,
    mMin: 0,
    mCanvasType: "bar",
    selfRoundRect: function (x, y, width, height, radius, fill, stroke) {
        if (typeof stroke == "undefined") {
            stroke = true;
        }
        if (typeof radius === "undefined") {
            radius = 5;
        }
        this.ctx.beginPath();
        this.ctx.moveTo(x + radius, y);
        this.ctx.lineTo(x + width - radius, y);
        this.ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
        this.ctx.lineTo(x + width, y + height - radius);
        this.ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
        this.ctx.lineTo(x + radius, y + height);
        this.ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
        this.ctx.lineTo(x, y + radius);
        this.ctx.quadraticCurveTo(x, y, x + radius, y);
        this.ctx.closePath();
        if (stroke) {
            this.ctx.stroke();
        }
        if (fill) {
            this.ctx.fill();
        }
    },
    drawSolidLine: function (startX, startY, endX, endY, color) {
        this.ctx.beginPath();
        this.ctx.moveTo(startX, startY);
        this.ctx.lineTo(endX, endY);
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = undefined == color ? this.color : color;
        this.ctx.stroke();
        this.ctx.closePath();
    },

    drawDashedLine: function (startX, startY, endX, endY, lineWidth, space) {
        this.ctx.beginPath();
        var len = endX - startX;
        var times = len / (lineWidth + space);
        var nextX;
        for (var i = 0; i < times; ++i) {
            nextX = startX + (lineWidth + space) * i;
            this.ctx.moveTo(nextX, startY);
            this.ctx.lineTo(nextX + lineWidth, startY);
        }
        this.ctx.lineWidth = 1; //线的粗度
        this.ctx.strokeStyle = "#DFDFDF"; // 红色路径
        this.ctx.stroke();
        this.ctx.closePath();
    },
    clear: function () {
        this.ctx.clearRect(0, 0, this.width, this.height);
        //解决某些机子上 不能清屏的BUG
        this.ctx.fillStyle = "#FFF";
        this.ctx.fillRect(0, 0, this.width, this.height);
    },
    clearRect: function () {
        this.clear();
    },
    draw: function (dataList, curFund) {
        this.mCanvasType = "bar";
        this.dataList = dataList;
        this.clear();
        if (!curFund) {
            this.roundRect();
        }
        this.mBarCount = dataList.length;
        var leftHeight = this.height - this.mTopPadding - this.mBottomPadding;
        var leftWidth = this.width - this.mLeftRightPadding * 2;
        var endLineX = this.width - this.mLeftRightPadding;
        var endLineY = this.height - this.mBottomPadding;
        var startX = this.mLeftRightPadding;
        var endY = this.mTopPadding;
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        endY = this.mTopPadding + leftHeight / 2
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        endY = this.height - this.mBottomPadding;
        this.drawSolidLine(startX, endY, endLineX, endY);

        var halfHeight = leftHeight / 2;
        var singleWidth = 60;// (leftWidth - (this.mBarCount + 1) * this.mBarMargin) / this.mBarCount;

        var nextBarX;
        var txtWidth;
        var barBottomTxtWidth;
        var barStartX = this.mLeftRightPadding;
        var itemGap = (leftWidth - (singleWidth * this.mBarCount)) / (this.mBarCount + 1)//this.mBarMargin;

        var max = -1000000;
        var min = 1000000;

        $.each(dataList, function (i, el) {
            if (!isNaN(el.value)) {
                if (parseFloat(el.value) > max) {
                    max = el.value;
                }
                if (parseFloat(el.value) < min) {
                    min = el.value;
                }
            }
        });

        var totalHeight = leftHeight + this.mTopExtra - halfHeight / 2;
        var barHeight;
        var ctx = this.ctx;
        var listSize = dataList.length;
        $.each(dataList, function (i, el) {
            nextBarX = barStartX + i * singleWidth + itemGap * (i + 1);
            if (max == min) {
                barHeight = totalHeight + halfHeight / 2;
            } else {
                barHeight = totalHeight * (el.value - min) / (max - min) + halfHeight / 2;
            }
            ctx.fillStyle = COLOR_LIST[i];
            if (curFund) {
                ctx.fillStyle = "#E2EBFA";
                if (el.value == min) {
                    ctx.fillStyle = "#80ADF7";
                } else if (el.value == max) {
                    ctx.fillStyle = "#FFD16C";
                }
            }
            ctx.fillRect(nextBarX, endLineY - barHeight, singleWidth, barHeight);
            //图形顶部文字
            ctx.font = "14px Arial";
            if (curFund) {
                ctx.font = "12px Arial";
            }
            ctx.fillStyle = "#FFF";
            if (curFund) {
                ctx.fillStyle = "#666";
                if ((el.value == min) || (el.value == max)) {
                    ctx.fillStyle = "#FFF";
                }
            }
            txtWidth = ctx.measureText(el.txt).width;
            ctx.fillText(el.txt, nextBarX + (singleWidth - txtWidth) / 2, endLineY - barHeight + 15, txtWidth);
            //图形底部文字
            ctx.font = "14px Arial";
            ctx.fillStyle = "#666";
            barBottomTxtWidth = ctx.measureText(el.barBottomTxt).width;
            ctx.fillText(el.barBottomTxt, nextBarX + (singleWidth - barBottomTxtWidth) / 2, endLineY + 20, singleWidth);
        });
    },
    drawTrend: function (dataList, max, min, isNeedDrawDate, topEl) {
        this.mCanvasType = "trend";
        this.clear();
        this.roundTrendRect();
        this.trendList = dataList;
        this.mMax = max;
        this.mMin = min;
        this.topEl = topEl;

        var ctx = this.ctx;
        var listSize = dataList.length;
        var leftHeight = this.height - (this.mTopPadding - 15) - this.mBottomPadding - this.topTouchItemHeight;
        var leftWidth = this.width - this.mLeftRightPadding * 2;
        var endLineX = this.width - this.mLeftRightPadding;
        var startX = this.mLeftRightPadding;
        var endY = (this.mTopPadding - 15) + this.topTouchItemHeight;
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        //左边最大刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(max[0], 2) + "%", startX, endY - 5);
        //右边最大刻度
        var txtWidth = this.ctx.measureText(max[1]).width;
        ctx.fillText(max[1], startX + leftWidth - txtWidth, endY - 5);

        endY = (this.mTopPadding - 15) + this.topTouchItemHeight + leftHeight / 2
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        endY = this.height - this.mBottomPadding;
        this.drawSolidLine(startX, endY, endLineX, endY);

        //左边最小刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(min[0], 2) + "%", startX, endY - 5);
        //右边最小刻度
        txtWidth = this.ctx.measureText(min[1]).width;
        ctx.fillText(min[1], startX + leftWidth - txtWidth, endY - 5);

        var bottomY = this.height - this.mBottomPadding;
        var gapAnnualizedYield = max[0] - min[0];
        var gapUnitYield = max[1] - min[1];

        var itemWidth = leftWidth / (listSize - 1);
        var baseStartX = this.mLeftRightPadding;
        var startY;
        //画七日年化
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfAnnualizedYield)) {
                var parserValue = parseFloat(el.MmfAnnualizedYield);
                startY = bottomY - leftHeight * (parserValue - min[0]) / gapAnnualizedYield;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#FE7E85";
        ctx.stroke();
        ctx.closePath();

        //画万份收益
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfUnitYield)) {
                var parserValue = parseFloat(el.MmfUnitYield);
                startY = bottomY - leftHeight * (parserValue - min[1]) / gapUnitYield;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });

        ctx.lineWidth = 1;
        ctx.strokeStyle = "#81ADF6";
        ctx.stroke();
        ctx.closePath();

        //X轴起始日期
        this.ctx.fillStyle = "#999";
        this.ctx.fillText(dataList[0].NavDate, baseStartX, bottomY + 20);
        //X轴截止日期
        var lastTxt = dataList[listSize - 1].NavDate;
        var txtWidth = this.ctx.measureText(lastTxt).width;
        this.ctx.fillText(lastTxt, baseStartX + leftWidth - txtWidth, bottomY + 20);

        //绘制顶部栏的交互信息,在手势交互的时候不需要绘制
        if (isNeedDrawDate) {
            if (this.topEl) {
                this.drawTopTxt(this.topEl, true);
            } else {
                this.drawTopTxt(dataList[listSize - 1]);
            }
        }
    },
    roundRect: function () {
        var ctx = this.ctx;
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#dfdfdf";
        this.selfRoundRect(1, 1, this.width - 2, this.height - 2, 5, true);
    },
    roundTrendRect: function () {
        var ctx = this.ctx;
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#dfdfdf";
        this.selfRoundRect(1, this.topTouchItemHeight - 1, this.width - 2, this.height - this.topTouchItemHeight - 2, 5, true);
    },
    reRender: function () {
        this.drawTrend(this.trendList, this.mMax, this.mMin, true, this.topEl);
    },
    drawTopTxt: function (element, unTouch) {
        var context = this.ctx;
        var defaultSub1 = "万份收益";
        var defaultSub2 = "七日年化";
        if (unTouch) {
            defaultSub1 = "累计万份收益";
            defaultSub2 = "平均七日年化";
        }
        var dataModel = [
            {
                rectColor: "#81ADF6",
                subTxt: defaultSub1,
                value: fixString(fixNum(element.MmfUnitYield, 4, true)),
                status: checkUpOrDown(element.MmfUnitYield),
                subTxtWidth: 0,
                valueWidth: 0
            },
            {
                rectColor: "#FE7E85",
                subTxt: defaultSub2,
                value: fixStringRate(fixNum(element.MmfAnnualizedYield, 2, true)),
                status: checkUpOrDown(element.MmfAnnualizedYield),
                subTxtWidth: 0,
                valueWidth: 0
            },
        ];

        //占据日期的位置
        var date = fixString(element.NavDate);
        var dateRectWidth = context.measureText(date).width;

        var cellRectWidth = parseInt(this.width / 3);
        if (date == "近一年") {
            cellRectWidth = parseInt(this.width * 0.22);
        }

        if ((cellRectWidth - 20) > dateRectWidth) {
            dateRectWidth = cellRectWidth - 20;
        } else {
            dateRectWidth = cellRectWidth;
        }

        var txtMargin = 3;
        var rectWidth = 10;
        var totalWidth = 0;
        context.font = "12px Arial";


        $.each(dataModel, function (index, el) {
            el.subTxtWidth = context.measureText(el.subTxt).width;
            el.valueWidth = 30;
            totalWidth += el.subTxtWidth;
            totalWidth += el.valueWidth;
            totalWidth += 2 * txtMargin;
            totalWidth += rectWidth;
        });

        var startY = 10;
        var startX = 0;
        //绘制日期
        context.fillStyle = "#999";
        context.fillText(date, 0, startY + 4, dateRectWidth);

        startX = dateRectWidth;
        var modelMargin = (this.width - totalWidth - dateRectWidth) / 2;

        $.each(dataModel, function (index, el) {
            //绘制矩形
            context.fillStyle = el.rectColor;
            context.fillRect(startX, startY - 6, rectWidth, rectWidth);

            //绘制文字
            startX += (rectWidth + txtMargin);
            context.fillStyle = "#999";
            context.fillText(el.subTxt, startX, startY + 4, el.subTxtWidth);

            startX += el.subTxtWidth + txtMargin;

            //根据涨跌绘制文字
            if (el.status == STATUS[0]) {
                context.fillStyle = "#ef3449";
            } else if (el.status == STATUS[2]) {
                context.fillStyle = "#0abf81";
            }
            context.fillText(el.value, startX, startY + 4, el.valueWidth);
            startX += el.valueWidth + modelMargin;
        });
    },
    drawReticle: function (pointX) {
        this.clear();
        var listSize = this.trendList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.mLeftRightPadding;
        var itemWidth = leftWidth / (listSize - 1);
        var i = 0;
        var find = false;
        for (; i < listSize; i++) {
            if (pointX < (itemWidth * i)) {
                find = true;
                break;
            }
        }
        var lastIndex = 0;
        if (find) {
            lastIndex = i;
        } else {
            lastIndex = listSize - 1;
        }

        //靠近i
        if ((itemWidth * i - (pointX - this.mLeftRightPadding)) > itemWidth / 2) {
            lastIndex = i - 1;
        }
        var truthStartX = this.mLeftRightPadding + lastIndex * itemWidth;
        var context = this.ctx;
        this.drawTrendWithOut(this.trendList, this.mMax, this.mMin);
        this.drawSolidLine(truthStartX, this.topTouchItemHeight, truthStartX, this.height - this.mBottomPadding, "#fcc84b");

        //画底部日期
        var baseStartX = this.mLeftRightPadding;
        var bottomY = this.height - this.mBottomPadding;
        //画底部日期
        var drawTxt = subFixedString(fixString(this.trendList[lastIndex].NavDate));
        var txtWidth = this.ctx.measureText(drawTxt).width;
        this.ctx.fillStyle = "#fcc84b";

        var tempStartX = 0;
        if (0 == lastIndex) {
            tempStartX = baseStartX;
        } else if (lastIndex == listSize - 1) {
            tempStartX = baseStartX + leftWidth - txtWidth;
        } else {
            tempStartX = truthStartX - txtWidth / 2;
            //是否超过左边界
            if (tempStartX < baseStartX) {
                tempStartX = baseStartX;
            }
            //是否超过右边界
            if ((truthStartX + txtWidth) > ( baseStartX + leftWidth)) {
                tempStartX = baseStartX + leftWidth - txtWidth;
            }
        }

        this.ctx.fillStyle = "#FFF8EE";
        this.ctx.strokeStyle = "#fcc84b";
        this.ctx.strokeRect(tempStartX - 2, bottomY + 5, txtWidth + 4, 20);
        this.ctx.fillRect(tempStartX - 2, bottomY + 5, txtWidth + 4, 20);

        this.ctx.fillStyle = "#fcc84b";
        this.ctx.fillText(drawTxt, tempStartX, bottomY + 20);

        this.drawTopTxt(this.trendList[lastIndex]);
    },
    drawTrendWithOut: function (dataList, max, min) {
        this.clear();
        this.roundTrendRect();
        this.trendList = dataList;
        this.mMax = max;
        this.mMin = min;

        var ctx = this.ctx;
        var listSize = dataList.length;
        var leftHeight = this.height - (this.mTopPadding - 15) - this.mBottomPadding - this.topTouchItemHeight;
        var leftWidth = this.width - this.mLeftRightPadding * 2;
        var endLineX = this.width - this.mLeftRightPadding;
        var startX = this.mLeftRightPadding;
        var endY = (this.mTopPadding - 15) + this.topTouchItemHeight;
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        //左边最大刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(max[0], 2) + "%", startX, endY - 5);
        //右边最大刻度
        var txtWidth = this.ctx.measureText(max[1]).width;
        ctx.fillText(max[1], startX + leftWidth - txtWidth, endY - 5);

        endY = (this.mTopPadding - 15) + this.topTouchItemHeight + leftHeight / 2
        this.drawDashedLine(startX, endY, endLineX, endY, 4, 2);
        endY = this.height - this.mBottomPadding;
        this.drawSolidLine(startX, endY, endLineX, endY, 4, 2);

        //左边最小刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(min[0], 2) + "%", startX, endY - 5);
        //右边最小刻度
        txtWidth = this.ctx.measureText(min[1]).width;
        ctx.fillText(min[1], startX + leftWidth - txtWidth, endY - 5);

        var bottomY = this.height - this.mBottomPadding;

        var gapAnnualizedYield = max[0] - min[0];
        var gapUnitYield = max[1] - min[1];

        var itemWidth = leftWidth / (listSize - 1);
        var baseStartX = this.mLeftRightPadding;
        var startY;
        //画七日年化
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfAnnualizedYield)) {
                var parserValue = parseFloat(el.MmfAnnualizedYield);
                startY = bottomY - leftHeight * (parserValue - min[0]) / gapAnnualizedYield;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#FE7E85";
        ctx.stroke();
        ctx.closePath();

        //画万份收益
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfUnitYield)) {
                var parserValue = parseFloat(el.MmfUnitYield);
                startY = bottomY - leftHeight * (parserValue - min[1]) / gapUnitYield;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });

        ctx.lineWidth = 1;
        ctx.strokeStyle = "#81ADF6";
        ctx.stroke();
        ctx.closePath();

        //X轴起始日期
        this.ctx.fillStyle = "#999";
        this.ctx.fillText(dataList[0].NavDate, baseStartX, bottomY + 20);
        //X轴截止日期
        var lastTxt = dataList[listSize - 1].NavDate;
        var txtWidth = this.ctx.measureText(lastTxt).width;
        this.ctx.fillText(lastTxt, baseStartX + leftWidth - txtWidth, bottomY + 20);
    },

};
