/**
 * Created by hyu.Yale on 2015/10/26.
 */
function TrendDrawer(el, callback) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }
    if (!this.canvas) {
        return;
    }
    this.ctx = this.canvas.getContext("2d");
    this.width = document.body.clientWidth * 0.96;
    this.height = this.canvas.height;
    this.callback = callback;

    var ctx = this.ctx,
        width = this.width,
        height = this.height;

    if (window.devicePixelRatio) {
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
        this.canvas.height = height * window.devicePixelRatio;
        this.canvas.width = width * window.devicePixelRatio;
        ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
    }
}

TrendDrawer.prototype = {
    width: 320,
    height: 195,
    version: "1.0.1",
    color: "#DFDFDF",
    tbPadding: 5,
    topHeight: 40,
    topTouchItemHeight: 30,
    bottomHeight: 35,
    leftRightMargin: 10,
    mIsNeedDrawDate: true,
    dataList: [],
    mMax: 0,
    mMin: 0,
    callback: {},
    mType: "unknow",
    topEl: undefined,
    selfRoundRect: function (x, y, width, height, radius, fill, stroke) {
        if (typeof stroke == "undefined") {
            stroke = true;
        }
        if (typeof radius === "undefined") {
            radius = 5;
        }
        this.ctx.beginPath();
        this.ctx.moveTo(x + radius, y);
        this.ctx.lineTo(x + width - radius, y);
        this.ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
        this.ctx.lineTo(x + width, y + height - radius);
        this.ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
        this.ctx.lineTo(x + radius, y + height);
        this.ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
        this.ctx.lineTo(x, y + radius);
        this.ctx.quadraticCurveTo(x, y, x + radius, y);
        this.ctx.closePath();
        if (stroke) {
            this.ctx.stroke();
        }
        if (fill) {
            this.ctx.fill();
        }
    },
    test: function (pointX) {
        this.drawSolidLine(pointX, 0, pointX, this.height, "red");
    },
    drawReticle: function (pointX, pointY) {
        this.clearCanvas();
        var listSize = this.dataList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.leftRightMargin;
        var itemWidth = leftWidth / (listSize - 1);
        var i = 0;
        var find = false;
        for (; i < listSize; i++) {
            if (pointX < (itemWidth * i)) {
                find = true;
                break;
            }
        }
        var lastIndex = 0;
        if (find) {
            lastIndex = i;
        } else {
            lastIndex = listSize - 1;
        }

        //靠近i
        if ((itemWidth * i - (pointX - this.leftRightMargin)) > itemWidth / 2) {
            lastIndex = i - 1;
        }
        var truthStartX = this.leftRightMargin + lastIndex * itemWidth;

        this.drawTrendWithOut(this.mType, this.dataList, this.mMax, this.mMin);
        this.drawSolidLine(truthStartX, this.topTouchItemHeight, truthStartX, this.height - this.bottomHeight + this.tbPadding, "#fcc84b");

        //画底部日期
        var baseStartX = this.leftRightMargin;
        var bottomY = this.height - this.bottomHeight;
        //画底部日期
        var drawTxt = subFixedString(fixString(this.dataList[lastIndex].NavDate));
        var txtWidth = this.ctx.measureText(drawTxt).width;
        this.ctx.fillStyle = "#fcc84b";

        var tempStartX = 0;
        if (0 == lastIndex) {
            tempStartX = baseStartX;
        } else if (lastIndex == listSize - 1) {
            tempStartX = baseStartX + leftWidth - txtWidth;
        } else {
            tempStartX = truthStartX - txtWidth / 2;
            //是否超过左边界
            if (tempStartX < baseStartX) {
                tempStartX = baseStartX;
            }
            //是否超过右边界
            if ((truthStartX + txtWidth) > ( baseStartX + leftWidth)) {
                tempStartX = baseStartX + leftWidth - txtWidth;
            }
        }

        this.ctx.fillStyle = "#FFF8EE";
        this.ctx.strokeStyle = "#fcc84b";
        this.ctx.strokeRect(tempStartX - 2, bottomY + 8, txtWidth + 4, 22);
        this.ctx.fillRect(tempStartX - 2, bottomY + 8, txtWidth + 4, 22);

        this.ctx.fillStyle = "#fcc84b";
        this.ctx.fillText(drawTxt, tempStartX, bottomY + 24);

        this.drawTopTxt(this.dataList[lastIndex]);
    },
    setTopEl: function (topEl) {
        this.topEl = topEl;
    },
    reRender: function () {
        if (this.mType && this.dataList && this.dataList.length > 0) {
            this.drawTrend(this.mType, this.dataList, this.mMax, this.mMin, true, this.topEl);
        }
    },
    roundRect: function () {
        var ctx = this.ctx;
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#dfdfdf";
        this.selfRoundRect(1, this.topTouchItemHeight, this.width - 2, this.height - this.topTouchItemHeight - 2, 5, true);
    },
    drawTopTxt: function (element) {
        var context = this.ctx;
        var dataModel = [
            {
                rectColor: "#fcc84b",
                subTxt: "本基金",
                value: fixStringRate(fixNum(element.Nav, 2, true)),
                status: checkUpOrDown(element.Nav),
                subTxtWidth: 0,
                valueWidth: 0,
                totalWidth: 0
            },
            {
                rectColor: "#82aef2",
                subTxt: "同类基金",
                value: fixStringRate(fixNum(element.SimFundAvgReturn, 2, true)),
                status: checkUpOrDown(element.SimFundAvgReturn),
                subTxtWidth: 0,
                valueWidth: 0,
                totalWidth: 0
            },
            {
                rectColor: "#ff7e84",
                subTxt: "沪深300",
                value: fixStringRate(fixNum(element.Hs300AvgReturn, 2, true)),
                status: checkUpOrDown(element.Hs300AvgReturn),
                subTxtWidth: 0,
                valueWidth: 0,
                totalWidth: 0
            }];

        var fontSize = 11;
        var txtMargin = 3;
        var rectWidth = 10;
        var totalWidth = 0;

        var startY = 10;
        var startX = 0;
        var offSetVertical = 9;
        var rectY = startY;

        if (document.body.clientWidth > 320) {
            rectWidth = 10;
            fontSize = 12;
            rectY = startY;
            //if (browser.versions.ios) {//ios终端
            //    rectY = startY + 2;
            //}
        }

        if (document.body.clientWidth > 360) {
            rectWidth = 10;
            fontSize = 12;
            rectY = startY;
        }

        $.each(dataModel, function (index, el) {
            context.font = (fontSize + "px Arial");
            el.subTxtWidth = context.measureText(el.subTxt).width;
            el.valueWidth = context.measureText("135.31%").width;
            el.totalWidth += (el.subTxtWidth + el.valueWidth + rectWidth + 2 * txtMargin);
            totalWidth += el.totalWidth;
        });

        //$("#xiaoming").html(document.body.clientWidth);
        var cellWidth = this.width / 3;

        $.each(dataModel, function (index, el) {
            startX = index * cellWidth;
            //绘制矩形
            context.fillStyle = el.rectColor;
            context.fillRect(startX, rectY, rectWidth, rectWidth);

            //绘制名称文字
            startX += (rectWidth + txtMargin);
            context.fillStyle = "#999";
            context.font = (fontSize + "px Arial");
            context.fillText(el.subTxt, startX, startY + offSetVertical, el.subTxtWidth);

            startX += el.subTxtWidth + txtMargin;
            //根据涨跌绘制文字
            if (el.status == STATUS[0]) {
                context.fillStyle = "#ef3449";
            } else if (el.status == STATUS[2]) {
                context.fillStyle = "#0abf81";
            }

            context.fillText(el.value, startX, startY + offSetVertical, el.valueWidth);
            //startX += el.valueWidth + modelMargin;


        });

        //context.fillStyle = "red";
        //context.fillRect(0, 0, this.width, 10);
    },

    drawTrend: function (type, dataList, max, min, isNeedDrawDate, topEl) {
        this.mType = type;
        this.clearCanvas();
        //绘制圆角矩形
        this.roundRect();
        this.dataList = dataList;
        this.mMax = max;
        this.mMin = min;
        this.topEl = topEl;

        var ctx = this.ctx;
        var listSize = dataList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.leftRightMargin;
        //可用高度
        var leftHeight = this.height - this.topTouchItemHeight - this.topHeight - this.bottomHeight;
        //每个点的横坐标
        var gap = max - min;
        var itemWidth = leftWidth / (listSize - 1);

        var startX = 0;
        var startY = 0;

        var baseStartX = this.leftRightMargin;
        var bottomY = this.height - this.bottomHeight;

        //画顶部的虚线
        this.drawDashedLine(baseStartX, this.topTouchItemHeight + this.topHeight - this.tbPadding, baseStartX + leftWidth, this.topHeight - this.tbPadding, 4, 2);
        ////画底部的虚线
        //var middleY = bottomY - leftHeight * (0 - min) / gap;
        //this.drawDashedLine(baseStartX, middleY, baseStartX + leftWidth, middleY, 5, 5);

        //画中线文字以及中线
        ctx.font = "11px Arial";
        ctx.fillStyle = "#999";
        var tempY = bottomY - leftHeight * (fixNum((max + min) / 2, 2) - min) / gap;
        ctx.fillText(fixStringRate(fixNum((max + min) / 2, 2)), baseStartX, tempY - 3);
        this.drawDashedLine(baseStartX, tempY, baseStartX + leftWidth, tempY, 4, 2);

        //画底部刻度
        startY = bottomY;
        ctx.fillText(fixNum(min, 2) + "%", baseStartX, startY);

        //画顶部刻度
        startY = this.topTouchItemHeight + this.topHeight - 2 * this.tbPadding;
        ctx.fillText(fixNum(max, 2) + "%", baseStartX, startY);

        //画下面的线
        this.drawSolidLine(baseStartX, bottomY + this.tbPadding, baseStartX + leftWidth, bottomY + this.tbPadding);

        //if (isNeedDrawDate) {
        //X轴起始日期
        var drawTxt = subFixedString(fixString(dataList[0].NavDate));
        this.ctx.fillStyle = "#999";
        this.ctx.fillText(drawTxt, baseStartX, bottomY + 24);

        //X轴截止日期
        drawTxt = subFixedString(fixString(dataList[listSize - 1].NavDate));
        var txtWidth = this.ctx.measureText(drawTxt).width;
        this.ctx.fillText(drawTxt, baseStartX + leftWidth - txtWidth, bottomY + 24);
        //}




        //画同类基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.SimFundAvgReturn)) {
                var parserValue = parseFloat(el.SimFundAvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#82aef2";
        ctx.stroke();
        ctx.closePath();

        //画沪深300基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Hs300AvgReturn)) {
                var parserValue = parseFloat(el.Hs300AvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#ff7e84";
        ctx.stroke();
        ctx.closePath();

        //画本基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Nav)) {
                var parserValue = parseFloat(el.Nav);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 2;
        ctx.strokeStyle = "#fcc84b";
        ctx.stroke();
        ctx.closePath();


        //绘制顶部栏的交互信息,在手势交互的时候不需要绘制
        if (isNeedDrawDate) {
            if (this.topEl) {
                this.drawTopTxt(this.topEl);
            } else {
                this.drawTopTxt(dataList[listSize - 1]);
            }
        }
    },
    drawTrendWithOut: function (type, dataList, max, min, isNeedDrawDate) {
        this.mType = type;
        this.clearCanvas();
        //绘制圆角矩形
        this.roundRect();
        this.dataList = dataList;
        this.mMax = max;
        this.mMin = min;

        var ctx = this.ctx;
        var listSize = dataList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.leftRightMargin;
        //可用高度
        var leftHeight = this.height - this.topTouchItemHeight - this.topHeight - this.bottomHeight;
        //每个点的横坐标
        var gap = max - min;
        var itemWidth = leftWidth / (listSize - 1);

        var startX = 0;
        var startY = 0;

        var baseStartX = this.leftRightMargin;
        var bottomY = this.height - this.bottomHeight;

        //画顶部的虚线
        this.drawDashedLine(baseStartX, this.topTouchItemHeight + this.topHeight - this.tbPadding, baseStartX + leftWidth, this.topHeight - this.tbPadding, 4, 2);

        //画中线文字以及中线
        ctx.font = "11px Arial";
        ctx.fillStyle = "#999";
        var tempY = bottomY - leftHeight * (fixNum((max + min) / 2, 2) - min) / gap;
        ctx.fillText(fixStringRate(fixNum((max + min) / 2, 2)), baseStartX, tempY - 3);
        this.drawDashedLine(baseStartX, tempY, baseStartX + leftWidth, tempY, 4, 2);

        //画底部刻度
        startY = bottomY;
        ctx.fillText(fixNum(min, 2) + "%", baseStartX, startY);
        //画顶部刻度
        startY = this.topTouchItemHeight + this.topHeight - 2 * this.tbPadding;
        ctx.fillText(fixNum(max, 2) + "%", baseStartX, startY);

        //画下面的线
        this.drawSolidLine(baseStartX, bottomY + this.tbPadding, baseStartX + leftWidth, bottomY + this.tbPadding);

        //if (isNeedDrawDate) {
        //X轴起始日期
        var drawTxt = subFixedString(fixString(dataList[0].NavDate));
        this.ctx.fillStyle = "#999";
        this.ctx.fillText(drawTxt, baseStartX, bottomY + 24);

        //X轴截止日期
        drawTxt = subFixedString(fixString(dataList[listSize - 1].NavDate));
        var txtWidth = this.ctx.measureText(drawTxt).width;
        this.ctx.fillText(drawTxt, baseStartX + leftWidth - txtWidth, bottomY + 24);
        //}


        //画同类基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.SimFundAvgReturn)) {
                var parserValue = parseFloat(el.SimFundAvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#82aef2";
        ctx.stroke();
        ctx.closePath();

        //画沪深300基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Hs300AvgReturn)) {
                var parserValue = parseFloat(el.Hs300AvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#ff7e84";
        ctx.stroke();
        ctx.closePath();

        //画本基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Nav)) {
                var parserValue = parseFloat(el.Nav);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 2;
        ctx.strokeStyle = "#fcc84b";
        ctx.stroke();
        ctx.closePath();
    },

    drawSolidLine: function (startX, startY, endX, endY, color) {
        this.ctx.beginPath();
        this.ctx.moveTo(startX, startY);
        this.ctx.lineTo(endX, endY);
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = undefined == color ? this.color : color;
        this.ctx.stroke();
        this.ctx.closePath();
    },
    drawDashedLine: function (startX, startY, endX, endY, lineWidth, space) {
        this.ctx.beginPath();
        var len = endX - startX;
        var times = len / (lineWidth + space);
        var nextX;
        for (var i = 0; i < times; ++i) {
            nextX = startX + (lineWidth + space) * i;
            this.ctx.moveTo(nextX, startY);
            this.ctx.lineTo(nextX + lineWidth, startY);
        }
        this.ctx.lineWidth = 1; //线的粗度
        this.ctx.strokeStyle = "#DFDFDF"; // 红色路径
        this.ctx.stroke();
        this.ctx.closePath();
    },
    clearCanvas: function () {
        this.ctx.clearRect(0, 0, this.width, this.height);
        //解决某些机子上 不能清屏的BUG
        this.ctx.fillStyle = "#FFF";
        this.ctx.fillRect(0, 0, this.width, this.height);
    },
}