/**
 * Created by hyu.Yale on 2015/10/26.
 */

function AssetsBarDrawer(el) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }
    if (!this.canvas) {
        return;
    }
    this.ctx = this.canvas.getContext("2d");
    this.mWidth = document.body.clientWidth;
    this.mHeight = this.canvas.height;

    this.mCenterX = this.canvas.width / 2;
    this.mDiameter = this.mModelDiameter * this.canvas.width / this.mModelBodyWidth;
    this.mSingleGap = this.mModelSingleGap * this.canvas.width / this.mModelBodyWidth;
    this.mRingWidth = this.mModelRingWidth * this.canvas.width / this.mModelBodyWidth;

    this.mCenterX = this.canvas.width / 2
    this.mCenterY = this.mSingleGap + this.mDiameter / 2;
}

AssetsBarDrawer.prototype = {
    mWidth: 0,//Canvas宽度
    mHeight: 0,//Canvas 高度
    mLeftMargin: 10,
    mRightMargin: 10,
    mScreenWidth: 320,//真实屏幕宽
    mModelTitleHeight: 50,//效果图上的标题栏高度
    mModelVerticalMargin: 32,//效果图上的上下间距
    mModelScreenWidth: 760,//效果图基于屏幕760宽度做的图
    mTitleHeight: 31,
    version: "1.0.1",
    color: "#DFDFDF",

    drawSolidLine: function (startX, startY, endX, endY) {
        var ctx = this.ctx;
        ctx.beginPath();
        ctx.moveTo(startX, startY);
        ctx.lineTo(endX, endY);
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#DFDFDF";
        ctx.stroke();
        ctx.closePath();
    },
    drawItem: function (dataList) {
        var ctx = this.ctx;

        var titleHeight = this.mTitleHeight;
        var graphLeftMargin = 22;
        var graphRightMargin = 18;

        var verMargin = 16;
        var itemHeight = 20;
        var startX = this.mLeftMargin;
        var startY = 0;

        var maxTxtWidth = 0;
        var measuredObj;
        var measuredRightObj;
        var rightTxtMaxWidth = 0;
        //找出最长的文字
        for (var i = 0; i < dataList.length; ++i) {
            measuredObj = ctx.measureText(dataList[i].name);

            if (measuredObj.width > maxTxtWidth) {
                maxTxtWidth = measuredObj.width;
            }

            measuredRightObj = ctx.measureText(dataList[i].rate + "%");
            if (measuredRightObj.width > rightTxtMaxWidth) {
                rightTxtMaxWidth = measuredRightObj.width;
            }
        }

        ctx.font = "15px Arial";
        ctx.fillStyle = "#333";
        var measureRightTitle = ctx.measureText("近三月涨幅");
        rightTxtMaxWidth = measureRightTitle.width > rightTxtMaxWidth ? measureRightTitle.width : rightTxtMaxWidth;
        var rectWidth = this.mWidth - graphLeftMargin - graphRightMargin - maxTxtWidth - rightTxtMaxWidth - this.mLeftMargin - this.mRightMargin;

        ctx.fillText("股票", this.mLeftMargin, 20);
        var titleStartX = this.mLeftMargin + maxTxtWidth + graphLeftMargin;
        ctx.fillText("占净值比", titleStartX, 20);
        titleStartX = this.mWidth - this.mRightMargin - rightTxtMaxWidth;
        ctx.fillText("近三月涨幅", titleStartX, 20);
        //画底部线
        this.drawSolidLine(0, titleHeight, this.mWidth, titleHeight);

        var childRectWidth = 0;
        var canvasWidth = this.mWidth;
        var canvasRightMargin = this.mRightMargin;

        //绘制文字名称

        for (var i = 0, tempMeasureTxt = ""; i < dataList.length; ++i) {
            startY = 15 + titleHeight + verMargin * (i + 1) + itemHeight * i;
            ctx.font = "15px Arial";
            ctx.fillStyle = "#666";
            ctx.fillText(dataList[i].name, startX, startY - 5);

            ////绘制背景灰色矩形
            ctx.fillStyle = "#f2f2f2";
            ctx.fillRect(this.mLeftMargin + maxTxtWidth + graphLeftMargin, startY - itemHeight, rectWidth, itemHeight);

            //绘制占比矩形
            ctx.fillStyle = "#fec74b";
            if (!isNaN(dataList[i].rate)) {
                childRectWidth = rectWidth * dataList[i].rate / 100;
                ctx.fillRect(this.mLeftMargin + maxTxtWidth + graphLeftMargin, startY - itemHeight, childRectWidth, itemHeight);
            }

            //绘制占比文字
            ctx.fillStyle = "#fff";
            ctx.font = "15px Arial";
            ctx.fillText(fixStringRate(fixNum(dataList[i].rate, 2)), 5 + this.mLeftMargin + maxTxtWidth + graphLeftMargin, startY - 5);


            //绘制右边的文字
            if (dataList[i].up) {
                ctx.fillStyle = "#ef3449";
            } else {
                ctx.fillStyle = "#0abf81";
            }
            tempMeasureTxt = fixStringRate(fixNum(dataList[i].rate, 2));
            ctx.fillText(fixStringRate(fixNum(dataList[i].rate, 2)), this.mWidth - this.mRightMargin - ctx.measureText(tempMeasureTxt).width, startY - 5);
        }

        //绘制涨跌图标
        var rateUpImg = new Image();
        rateUpImg.src = "style/images/arrow_up.png";
        var imgStartX = canvasWidth - canvasRightMargin - rightTxtMaxWidth - graphRightMargin - 12;
        if (rateUpImg.complete) {
            for (var i = 0; i < dataList.length; ++i) {
                if (dataList[i].up) {
                    startY = 25 + verMargin * (i + 1) + itemHeight * i;
                    ctx.drawImage(rateUpImg, imgStartX, startY);
                }
            }
        } else {
            rateUpImg.onload = function () {
                for (var i = 0; i < dataList.length; ++i) {
                    if (dataList[i].up) {
                        startY = 25 + verMargin * (i + 1) + itemHeight * i;
                        ctx.drawImage(rateUpImg, imgStartX, startY);
                    }
                }
            }
        }

        var rateDownImg = new Image();
        rateDownImg.src = "style/images/arrow_down.png";
        if (rateDownImg.complete) {
            for (var i = 0; i < dataList.length; ++i) {
                if (false == dataList[i].up) {
                    startY = 25 + verMargin * (i + 1) + itemHeight * i;
                    ctx.drawImage(rateDownImg, imgStartX, startY);
                }
            }
        } else {
            rateDownImg.onload = function () {
                for (var i = 0; i < dataList.length; ++i) {
                    if (false == dataList[i].up) {
                        startY = 25 + verMargin * (i + 1) + itemHeight * i;
                        ctx.drawImage(rateDownImg, imgStartX, startY);
                    }
                }
            }
        }
    },

    draw: function (dataList) {
        var validDataList = [];
        //校验有效数据
        $.each(dataList, function (index, el) {
            if (!isEmpty(el.name)) {
                validDataList.push({name: el.name, rate: el.rate, rateTxt: el.rateTxt, up: el.up})
            }
        });

        //dataList = undefined;
        if (validDataList && validDataList.length > 0) {
            //重新设置canvas的高度
            this.mHeight = validDataList.length * 37 + this.mTitleHeight;
            $(".assetsbar-canvas-center").css("height", this.mHeight + 'px');
            $(".assetsbar-canvas-center").css("width", this.mWidth + 'px');

            var ctx = this.ctx,
                width = this.mWidth,
                height = this.mHeight;

            if (window.devicePixelRatio) {
                this.canvas.style.width = width + "px";
                this.canvas.style.height = height + "px";
                this.canvas.height = height * window.devicePixelRatio;
                this.canvas.width = width * window.devicePixelRatio;
                ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
            }

            this.drawItem(validDataList);
        } else {
            this.drawEmpty();
        }
    },
    drawEmpty: function () {
        this.mHeight = 70 + 22;
        $(".assetsbar-canvas-center").css("height", this.mHeight + 'px');
        $(".assetsbar-canvas-center").css("width", this.mWidth + 'px');

        var ctx = this.ctx,
            width = this.mWidth,
            height = this.mHeight;

        if (window.devicePixelRatio) {
            this.canvas.style.width = width + "px";
            this.canvas.style.height = height + "px";
            this.canvas.height = height * window.devicePixelRatio;
            this.canvas.width = width * window.devicePixelRatio;
            ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
        }

        var txt = "暂无数据";
        var measuredObj = this.ctx.measureText(txt);
        ctx.font = "15px Arial";
        this.ctx.fillStyle = "#666";
        var startX = (this.mWidth - measuredObj.width) / 2;
        ctx.fillText("暂无数据", startX, 50, 100);
    }
};