/**
 * Created by hyu.Yale on 2015/10/26.
 */

function ScaleDrawer(el) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }
    if (!this.canvas) {
        return;
    }
    this.ctx = this.canvas.getContext("2d");
    this.width = document.body.clientWidth * 0.9;
    this.height = this.canvas.height;
    var ctx = this.ctx,
        width = this.width,
        height = this.height;

    if (window.devicePixelRatio) {
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
        this.canvas.height = height * window.devicePixelRatio;
        this.canvas.width = width * window.devicePixelRatio;
        ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
    }
}

ScaleDrawer.prototype = {
    mWidth: 0,//Canvas宽度
    mHeight: 0,//Canvas 高度
    mBarCount: 4,
    mTopPadding: 40,
    mBottomPadding: 30,
    mLeftRightPadding: 10,
    mTopExtra: 10,
    mBarMargin: 20,
    color: "#DFDFDF",
    dataList:[],
    drawSolidLine: function (startX, startY, endX, endY) {
        this.ctx.beginPath();
        this.ctx.moveTo(startX, startY);
        this.ctx.lineTo(endX, endY);
        this.ctx.strokeStyle = this.color;
        this.ctx.lineWidth = 1;
        this.ctx.stroke();
        this.ctx.closePath();
    },

    drawDashedLine: function (startX, startY, endX, endY, lineWidth, space) {
        this.ctx.beginPath();
        var len = endX - startX;
        var times = len / (lineWidth + space);
        var nextX;
        for (var i = 0; i < times; ++i) {
            nextX = startX + (lineWidth + space) * i;
            this.ctx.moveTo(nextX, startY);
            this.ctx.lineTo(nextX + lineWidth, startY);
        }
        this.ctx.lineWidth = 1; //线的粗度
        this.ctx.strokeStyle = this.color; // 红色路径
        this.ctx.stroke();
        this.ctx.closePath();
    },
    clear: function () {
        this.ctx.clearRect(0, 0, this.width, this.height);
    },
    clearRect: function () {
        this.ctx.clearRect(0, 0, this.width, this.height);
    },
    draw: function (dataList) {
        this.dataList = dataList;
        this.ctx.clearRect(0, 0, this.width, this.height);
        this.mBarCount = dataList.length;
        var leftHeight = this.height - this.mTopPadding - this.mBottomPadding;
        var leftWidth = this.width - this.mLeftRightPadding * 2;
        var endLineX = this.width - this.mLeftRightPadding;
        var endLineY = this.height - this.mBottomPadding;
        var startX = this.mLeftRightPadding;
        var endY = this.mTopPadding;
        this.drawDashedLine(startX, endY, endLineX, endY, 15, 5);
        endY = this.mTopPadding + leftHeight / 2
        this.drawDashedLine(startX, endY, endLineX, endY, 15, 5);
        endY = this.height - this.mBottomPadding;
        this.drawSolidLine(startX, endY, endLineX, endY, 15, 5);

        var halfHeight = leftHeight / 2;
        var singleWidth = (leftWidth - (this.mBarCount + 1) * this.mBarMargin) / this.mBarCount;

        var nextBarX;
        var txtWidth;
        var barBottomTxtWidth;
        var barStartX = this.mLeftRightPadding;
        var itemGap = this.mBarMargin;

        var max = -1000000;
        var min = 1000000;

        $.each(dataList, function (i, el) {
            if (!isNaN(el.value)) {
                if (parseInt(el.value) > max) {
                    max = el.value;
                }
                if (parseInt(el.value) < min) {
                    min = el.value;
                }
            }
        });

        //针对单一条数据的优化
        //if (max = min) {
        //    min = 0;
        //    singleWidth = 50;
        //    itemGap = (leftWidth - singleWidth) / 2;
        //}

        var totalHeight = leftHeight + this.mTopExtra - halfHeight / 2;
        var barHeight;
        var ctx = this.ctx;
        $.each(dataList, function (i, el) {
            nextBarX = barStartX + i * singleWidth + itemGap * (i + 1);
            barHeight = totalHeight * (el.value - min) / (max - min) + halfHeight / 2;
            ctx.fillStyle = COLOR_LIST[i];
            ctx.fillRect(nextBarX, endLineY - barHeight, singleWidth, barHeight);
            //图形顶部文字
            txtWidth = ctx.measureText(el.txt).width;
            ctx.fillStyle = "#FFF";
            ctx.fillText(el.txt, nextBarX + (singleWidth - txtWidth) / 2, endLineY - barHeight + 15, txtWidth);
            //图形底部文字
            ctx.fillStyle = "#666";
            barBottomTxtWidth = ctx.measureText(el.barBottomTxt).width;
            ctx.fillText(el.barBottomTxt, nextBarX + (singleWidth - barBottomTxtWidth) / 2, endLineY + 20, singleWidth);
        });
    },
    drawTrend: function (dataList, max, min) {
        this.clear();
        var ctx = this.ctx;
        var listSize = dataList.length;
        var leftHeight = this.height - this.mTopPadding - this.mBottomPadding;
        var leftWidth = this.width - this.mLeftRightPadding * 2;
        var endLineX = this.width - this.mLeftRightPadding;
        var startX = this.mLeftRightPadding;
        var endY = this.mTopPadding;
        this.drawDashedLine(startX, endY, endLineX, endY, 15, 5);
        //左边最大刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(max[0], 2) + "%", startX, endY - 5);
        //右边最大刻度
        var txtWidth = this.ctx.measureText(max[1]).width;
        ctx.fillText(max[1], startX + leftWidth - txtWidth, endY - 5);

        endY = this.mTopPadding + leftHeight / 2
        this.drawDashedLine(startX, endY, endLineX, endY, 15, 5);
        endY = this.height - this.mBottomPadding;
        this.drawSolidLine(startX, endY, endLineX, endY, 15, 5);

        //左边最小刻度
        ctx.fillStyle = "#999";
        ctx.fillText(fixNum(min[0], 2) + "%", startX, endY - 5);
        //右边最小刻度
        txtWidth = this.ctx.measureText(min[1]).width;
        ctx.fillText(min[1], startX + leftWidth - txtWidth, endY - 5);

        var bottomY = this.height - this.mBottomPadding;
        var gap = max[0] - min[0];
        var itemWidth = leftWidth / (listSize - 1);
        var baseStartX = this.mLeftRightPadding;
        var startY;
        //画七日年化
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfAnnualizedYield)) {
                var parserValue = parseFloat(el.MmfAnnualizedYield);
                startY = bottomY - leftHeight * (parserValue - min[0]) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#fcc84b";
        ctx.stroke();
        ctx.closePath();


        //画万份收益
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.MmfUnitYield)) {
                var parserValue = parseFloat(el.MmfUnitYield);
                startY = bottomY - leftHeight * (parserValue - min[1]) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });

        ctx.lineWidth = 1;
        ctx.strokeStyle = "red";
        ctx.stroke();
        ctx.closePath();

        //X轴起始日期
        this.ctx.fillStyle = "#999";
        this.ctx.fillText(dataList[0].NavDate, baseStartX, bottomY + 20);
        //X轴截止日期
        var lastTxt = dataList[listSize - 1].NavDate;
        var txtWidth = this.ctx.measureText(lastTxt).width;
        this.ctx.fillText(lastTxt, baseStartX + leftWidth - txtWidth, bottomY + 20);
    }
}
