/**
 * Created by hyu.Yale on 2015/10/26.
 */

CanvasRenderingContext2D.prototype.roundRect = function (x, y, w, h, r) {
    //if (w < 2 * r) r = w / 2;
    //if (h < 2 * r) r = h / 2;
    this.beginPath();
    this.moveTo(x + r, y);
    this.arcTo(x + w, y, x + w, y + h, r);
    this.arcTo(x + w, y + h, x, y + h, r);
    this.arcTo(x, y + h, x, y, r);
    this.arcTo(x, y, x + w, y, r);
    // this.arcTo(x+r, y);
    this.closePath();
    return this;
}


function TrendDrawer(el, callback) {
    if (undefined == this.canvas) {
        this.canvas = document.getElementById(el);
    }
    if (!this.canvas) {
        return;
    }
    this.ctx = this.canvas.getContext("2d");
    this.width = document.body.clientWidth * 0.96;
    this.height = this.canvas.height;
    this.callback = callback;

    var ctx = this.ctx,
        width = this.width,
        height = this.height;

    if (window.devicePixelRatio) {
        this.canvas.style.width = width + "px";
        this.canvas.style.height = height + "px";
        this.canvas.height = height * window.devicePixelRatio;
        this.canvas.width = width * window.devicePixelRatio;
        ctx.scale(window.devicePixelRatio, window.devicePixelRatio);
    }
}

TrendDrawer.prototype = {
    width: 320,
    height: 195,
    version: "1.0.1",
    color: "#DFDFDF",
    tbPadding: 5,
    topHeight: 40,
    topTouchItemHeight: 30,
    bottomHeight: 30,
    leftRightMargin: 10,
    mIsNeedDrawDate: true,
    dataList: [],
    mMax: 0,
    mMin: 0,
    callback: {},
    mType: "unknow",
    drawReticle: function (pointX, pointY) {
        var listSize = this.dataList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.leftRightMargin;
        var itemWidth = leftWidth / (listSize - 1);
        var i = 0;
        for (; i < listSize; i++) {
            if (pointX < itemWidth * i) {
                break;
            }
        }
        var lastIndex = i;
        //靠近i
        if ((itemWidth * i - pointX) > itemWidth / 2) {
            lastIndex = i - 1;
        }
        var truthStartX = this.leftRightMargin + lastIndex * itemWidth;

        //if (this.callback) {
        //    this.callback(fixNum(this.dataList[lastIndex].Nav, 2) + "%", fixNum(this.dataList[lastIndex].SimFundAvgReturn, 2) + "%", fixNum(this.dataList[lastIndex].Hs300AvgReturn, 2) + "%");
        //}
        this.clear();
        this.drawTrend(this.mType, this.dataList, this.mMax, this.mMin, false);

        this.drawSolidLine(truthStartX, this.topTouchItemHeight, truthStartX, this.height - this.bottomHeight + this.tbPadding, "#fcc84b");

        //画底部日期
        var baseStartX = this.leftRightMargin;
        var bottomY = this.height - this.bottomHeight;
        //画底部日期
        var drawTxt = subFixedString(this.dataList[lastIndex].NavDate, 5);
        var txtWidth = this.ctx.measureText(drawTxt).width;

        if (0 == lastIndex) {
            this.ctx.fillText(drawTxt, baseStartX, bottomY + 22);
        } else if (lastIndex == listSize - 1) {
            this.ctx.fillText(drawTxt, baseStartX + leftWidth - txtWidth, bottomY + 22);
        } else {
            this.ctx.fillText(drawTxt, truthStartX - txtWidth / 2, bottomY + 22);
        }

        this.drawTopTxt(this.dataList[lastIndex]);
    },
    reRender: function () {
        this.drawTrend(this.mType, this.dataList, this.mMax, this.mMin, true);
    },
    roundRect: function () {
        var ctx = this.ctx;
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#999";
        ctx.roundRect(0, this.topTouchItemHeight, this.width - 1, this.height - this.topTouchItemHeight, 6).stroke();
    },

    drawTopTxt: function (element) {
        var context = this.ctx;

        //context.fillStyle = "red";
        //context.fillRect(0, 0, this.width, this.topTouchItemHeight);
        var dataModel = [
            {
                rectColor: "#fcc84b",
                subTxt: "本基金",
                value: fixStringRate(fixNum(element.Nav, 2)),
                status: checkUpOrDown(element.Nav),
                subTxtWidth: 0,
                valueWidth: 0
            },
            {
                rectColor: "#82aef2",
                subTxt: "同类基金",
                value: fixStringRate(fixNum(element.SimFundAvgReturn, 2)),
                status: checkUpOrDown(element.SimFundAvgReturn),
                subTxtWidth: 0,
                valueWidth: 0
            },
            {
                rectColor: "#ff7e84",
                subTxt: "沪深300",
                value: fixStringRate(fixNum(element.Hs300AvgReturn, 2)),
                status: checkUpOrDown(element.Hs300AvgReturn),
                subTxtWidth: 0,
                valueWidth: 0
            }];

        var txtMargin = 3;
        var rectWidth = 10;
        var totalWidth = 0;
        $.each(dataModel, function (index, el) {
            el.subTxtWidth = context.measureText(el.subTxt).width;
            el.valueWidth = context.measureText(el.value).width;
            totalWidth += el.subTxtWidth;
            totalWidth += el.valueWidth;
            totalWidth += 2 * txtMargin;
            totalWidth += rectWidth;
        });

        var modelMargin = (this.width - totalWidth) / 2;

        var startY = 10;
        var startX = 0;
        $.each(dataModel, function (index, el) {
            //绘制矩形
            context.fillStyle = el.rectColor;
            context.fillRect(startX, startY, rectWidth, rectWidth);

            //绘制文字
            startX += (rectWidth + txtMargin);
            context.fillStyle = "#999";
            context.fillText(el.subTxt, startX, startY + 9, el.subTxtWidth);

            startX += el.subTxtWidth + txtMargin;

            //根据涨跌绘制文字
            if (el.status == STATUS[0]) {
                context.fillStyle = "#ef3449";
            } else if (el.status == STATUS[2]) {
                context.fillStyle = "#0abf81";
            }
            context.fillText(el.value, startX, startY + 9, el.valueWidth);
            startX += el.valueWidth + modelMargin;
        });
    },

    drawTrend: function (type, dataList, max, min, isNeedDrawDate) {
        this.mType = type;
        this.clear();
        //绘制圆角矩形
        this.roundRect();
        this.dataList = dataList;
        this.mMax = max;
        this.mMin = min;

        var ctx = this.ctx;
        var listSize = dataList.length;
        //可用宽度
        var leftWidth = this.width - 2 * this.leftRightMargin;
        //可用高度
        var leftHeight = this.height - this.topTouchItemHeight - this.topHeight - this.bottomHeight;
        //每个点的横坐标
        var gap = max - min;
        var itemWidth = leftWidth / (listSize - 1);

        var startX = 0;
        var startY = 0;

        var baseStartX = this.leftRightMargin;
        var bottomY = this.height - this.bottomHeight;

        //画顶部的虚线
        this.drawDashedLine(baseStartX, this.topTouchItemHeight + this.topHeight - this.tbPadding, baseStartX + leftWidth, this.topHeight - this.tbPadding, 5, 5);
        //画中线的0线
        var middleY = bottomY - leftHeight * (0 - min) / gap;
        this.drawDashedLine(baseStartX, middleY, baseStartX + leftWidth, middleY, 5, 5);

        //画0线
        ctx.font = "11px Arial";
        ctx.fillStyle = "#999";
        ctx.fillText("0", baseStartX, middleY - this.tbPadding);
        //画底部刻度
        startY = bottomY;
        ctx.fillText(fixNum(min, 2) + "%", baseStartX, startY);
        //画顶部刻度
        startY = this.topTouchItemHeight + this.topHeight - 2 * this.tbPadding;
        ctx.fillText(fixNum(max, 2) + "%", baseStartX, startY);

        //画下面的线
        this.drawSolidLine(baseStartX, bottomY + this.tbPadding, baseStartX + leftWidth, bottomY + this.tbPadding);

        if (isNeedDrawDate) {
            //X轴起始日期
            var drawTxt = subFixedString(dataList[0].NavDate, 5);
            this.ctx.fillStyle = "#999";
            this.ctx.fillText(drawTxt, baseStartX, bottomY + 22);

            //X轴截止日期
            drawTxt = subFixedString(dataList[listSize - 1].NavDate, 5);
            var txtWidth = this.ctx.measureText(drawTxt).width;
            this.ctx.fillText(drawTxt, baseStartX + leftWidth - txtWidth, bottomY + 22);
        }

        //画本基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Nav)) {
                var parserValue = parseFloat(el.Nav);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#fcc84b";
        ctx.stroke();
        ctx.closePath();


        //画同类基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.SimFundAvgReturn)) {
                var parserValue = parseFloat(el.SimFundAvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#82aef2";
        ctx.stroke();
        ctx.closePath();

        //画沪深300基金走势线
        ctx.beginPath();
        $.each(dataList, function (index, el) {
            if (!isNaN(el.Hs300AvgReturn)) {
                var parserValue = parseFloat(el.Hs300AvgReturn);
                startY = bottomY - leftHeight * (parserValue - min) / gap;
                startX = baseStartX + index * itemWidth;
                if (0 == index) {
                    ctx.moveTo(startX, startY);
                } else {
                    ctx.lineTo(startX, startY);
                }
            }
        });
        ctx.lineWidth = 1;
        ctx.strokeStyle = "#ff7e84";
        ctx.stroke();
        ctx.closePath();

        //绘制顶部栏的交互信息,在手势交互的时候不需要绘制
        if (isNeedDrawDate) {
            this.drawTopTxt(dataList[listSize - 1]);
        }
    },
    drawSolidLine: function (startX, startY, endX, endY, color) {
        this.ctx.beginPath();
        this.ctx.moveTo(startX, startY);
        this.ctx.lineTo(endX, endY);
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = undefined == color ? this.color : color;
        this.ctx.stroke();
        this.ctx.closePath();
    },
    drawDashedLine: function (startX, startY, endX, endY, lineWidth, space) {
        this.ctx.beginPath();
        var len = endX - startX;
        var times = len / (lineWidth + space);
        var nextX;
        for (var i = 0; i < times; ++i) {
            nextX = startX + (lineWidth + space) * i;
            this.ctx.moveTo(nextX, startY);
            this.ctx.lineTo(nextX + lineWidth, startY);
        }
        this.ctx.lineWidth = 1; //线的粗度
        this.ctx.strokeStyle = this.color; // 红色路径
        this.ctx.stroke();
        this.ctx.closePath();
    },
    clear: function () {
        this.ctx.clearRect(0, 0, this.width, this.height);
    },
}